﻿require 'sketchup.rb'

module CMD
module Animate_Sections


	def self.upAnimate()

		
		model = Sketchup.active_model
		sel = model.selection
		grp =[]
		applyMessage = false
		
		if sel.length == 0; return UI.messagebox("Nothing Selected!\nPlease select a group/s you wish to apply sections to."); end;
		
		sel.each {|e|
			if e.is_a?(Sketchup::Group)
				grp << e; 
			else 
				applyMessage = true
			end
		}
		
		if grp.length == 0; return UI.messagebox("No groups have been selected!\nPlease select a group/s you wish to apply sections to."); end;
		
		if applyMessage
			result = UI.messagebox("\"Animate Up\" sections will only be applied to the Groups in your selection.\n\nDo you wish to continue?",MB_YESNO)
			if (result != IDYES)
				return;	
			end
		end
		
		
		
		prompts = ["Animation:"]
		defaults = ["BLANK"]
		input = UI.inputbox(prompts, defaults, "Name the Section Animation")
		if input == false
			return;
		else
			animName = input[0]
		end
	
		model.start_operation("Animate",true)
		
		layerName =  "Section: " + animName
	
		#apply "Bottom" section
		grp.each{|g|
			bb = g.bounds
	
			pMin = bb.min
			bZ = pMin.z - 1
			pMin.z = bZ
	
			vec = Geom::Vector3d.new(0,0,-1)

			btmSP = g.entities.add_section_plane([pMin, vec])
			btmSP.activate
		
			layers = model.layers.add layerName
			btmSP.layer = layerName

		}

		pageName = animName + "_Bottom"
		page = model.pages
		btmPage = page.add pageName
		status = btmPage.use_camera = false
		status = btmPage.use_axes = false
		status = btmPage.use_hidden = false
		status = btmPage.use_rendering_options = false
		status = btmPage.use_section_planes = true
		status = btmPage.set_visibility layerName, true
		status = btmPage.use_shadow_info = false
		status = btmPage.use_style = false
	
	
	
		#apply "Top" section
		grp.each{|g|
	
			bb = g.bounds
	
			pMax = bb.max
			tZ = pMax.z + 1
			pMax.z = tZ
	
			vec = Geom::Vector3d.new(0,0,-1)
			topSP = g.entities.add_section_plane([pMax, vec])
			topSP.activate
	
			layers = model.layers.add layerName
			topSP.layer = layerName
	
		}
	
		pageName = animName + "_Top"
		page = model.pages
		btmPage = page.add  pageName
		status = btmPage.use_camera = false
		status = btmPage.use_axes = false
		status = btmPage.use_hidden = false
		status = btmPage.use_rendering_options = false
		status = btmPage.use_section_planes = true
		status = btmPage.set_visibility pageName, true
		status = btmPage.use_shadow_info = false
		status = btmPage.use_style = false
		
		#end "undo" functionality
		model.commit_operation
		
		
	end




	def self.vectorAnimate()
	
		model = Sketchup.active_model
		sel = model.selection
		grp =[]
		edg =[]
		msgEdg = ""
	
		sel.each {|e|

			if (e.is_a?(Sketchup::Edge)); edg << e; end
			if (e.is_a?(Sketchup::Group)); grp << e; end
		}
	
		eNum = edg.length
		gNum = grp.length
	
		unless (eNum == 1) && (gNum == 1)
			msgEdg = "Your selection: " + eNum.to_s + " Edges, " + gNum.to_s + " Groups\n\nSelect 1 Edge and 1 Group ONLY"; return UI.messagebox(msgEdg)
		end

		# Single "Undo" functionality
		model.start_operation("Animate",true)

		prompts = ["Animation:"]
		defaults = ["BLANK"]
		input = UI.inputbox(prompts, defaults, "Name the Section Animation")
		if input == false
			return;
		else
			animName = input[0]
		end
	
		layerName =  "Section: " + animName
	
		bb = grp[0].bounds	
	
		gTrans = grp[0].transformation
	
		gOrig = gTrans.origin
		l = edg[0].line
		vec = l[1]
		
		
		#apply "Start" section
		oX = gOrig.x
		oY = gOrig.y
		oZ = gOrig.z 
		sX = edg[0].start.position.x
		sY = edg[0].start.position.y
		sZ = edg[0].start.position.z
	
		sStart = Geom::Point3d.new((sX-oX),(sY-oY),(sZ-oZ))
	 
		stSP = grp[0].entities.add_section_plane([sStart,vec])
		stSP.activate

		layers = model.layers.add layerName
		stSP.layer = layerName

		page = model.pages
		stName = animName + "_Start"
		stPage = page.add stName
		status = stPage.use_camera = false
		status = stPage.use_axes = false
		status = stPage.use_hidden = false
		status = stPage.use_rendering_options = false
		status = stPage.use_section_planes = true
		status = stPage.set_visibility stName, true
		status = stPage.use_shadow_info = false
		status = stPage.use_style = false
	
		
		#apply "Finish" section
	
		eX = edg[0].end.position.x;
		eY = edg[0].end.position.y;
		eZ = edg[0].end.position.z;

		sEnd = Geom::Point3d.new((eX-oX),(eY-oY),(eZ-oZ))
	
		endSP = grp[0].entities.add_section_plane([sEnd,vec])
		endSP.activate
	
		layers = model.layers.add layerName
		endSP.layer = layerName
	
		page = model.pages
		endName = animName + "_Finish"
		endPage = page.add endName
		status = endPage.use_camera = false
		status = endPage.use_axes = false
		status = endPage.use_hidden = false
		status = endPage.use_rendering_options = false
		status = endPage.use_section_planes = true
		status = endPage.set_visibility endName, true
		status = endPage.use_shadow_info = false
		status = endPage.use_style = false
		
		#end "undo" functionality
		model.commit_operation
		
	end
	
	def self.loadToolbar

		toolbar = UI::Toolbar.new "Animate Sections"

		 cmd1 = UI::Command.new("Animate Up") { upAnimate }
		 cmd1.small_icon = "anim_sec_images/upBtn.jpg"
		 cmd1.large_icon = "anim_sec_images/upBtn.jpg"
		 cmd1.tooltip = "Animate Up"
		 cmd1.status_bar_text = "Animate Up"
		 cmd1.menu_text = "Animate Up"
		 toolbar = toolbar.add_item cmd1

	 
		 cmd2 = UI::Command.new("Animate by Vector") { vectorAnimate }
		 cmd2.small_icon = "anim_sec_images/vecBtn.jpg"
		 cmd2.large_icon = "anim_sec_images/vecBtn.jpg"
		 cmd2.tooltip = "Animate by Vector"
		 cmd2.status_bar_text = "Animate by Vector"
		 cmd2.menu_text = "Animate by Vector"
		 toolbar = toolbar.add_item cmd2
	 
	end
	
	unless file_loaded?(__FILE__)
	file_loaded(__FILE__)
		
		pc_menu= UI.menu("Plugins").add_submenu "Animate Sections"
		pc_menu.add_item("Animate Up"){ upAnimate }
		pc_menu.add_item("Animate by Vector"){ vectorAnimate }
	
		loadToolbar
	
	end
	
	
	
	
#end of modules
end 
end
